@@ -28,6 +28,7 @@ logging.getLogger('urllib3.connectionpool').setLevel(logging.CRITICAL)
 
 HOSTNAME = 'api.github.com'
 VERSION = 'v3'
+DEFAULT_PAGE_SIZE = 30
 
 
 class GitHubAPI(RESTclient):
@@ -55,19 +56,15 @@ class GitHubAPI(RESTclient):
         headers['Accept'] = f'application/vnd.github.{self.version}+json'
         return headers
 
-    def _get_next_endpoint(self, link_header):
-        """ return next endpoint from link header
+    def _get_next_endpoint(self, url):
+        """ return next endpoint
         """
-        if not link_header:
+        if not url:
             logger.debug('link header is empty')
             return
-        regex = fr".*<https://{self.hostname}(?P<endpoint>/.*?)>; rel=\"next\".*"
-        match = re.match(regex, link_header)
-        if match:
-            endpoint = match.group('endpoint')
-            logger.debug(f'found next endpoint in link header: {endpoint}')
-            return endpoint
-        logger.debug('next endpoints not found in link header')
+        endpoint = self.get_endpoint_from_url(url)
+        logger.debug(f'next endpoint is: {endpoint}')
+        return endpoint
 
     def _get_all(self, endpoint, **kwargs):
         """ return all pages from endpoint
@@ -75,7 +72,7 @@ class GitHubAPI(RESTclient):
         logger.debug(f'get items from: {endpoint}')
         items = []
         while True:
-            link_header = None
+            url = None
             response = super(GitHubAPI, self).get(endpoint, raw_response=True, **kwargs)
             if response:
                 data = response.json()
@@ -83,9 +80,9 @@ class GitHubAPI(RESTclient):
                     items.extend(response.json())
                 else:
                     items.append(data)
-                link_header = response.headers.get('Link')
+                url = response.links.get('next', {}).get('url')
 
-            endpoint = self._get_next_endpoint(link_header)
+            endpoint = self._get_next_endpoint(url)
             if not endpoint:
                 logger.debug('no more pages to retrieve')
                 break
@@ -97,9 +94,8 @@ class GitHubAPI(RESTclient):
         """
         while True:
             response = super(GitHubAPI, self).get(endpoint, raw_response=True, **kwargs)
-            for page in response.json():
-                yield page
-            endpoint = self._get_next_endpoint(response.headers.get('Link'))
+            yield response.json()
+            endpoint = self._get_next_endpoint(response.links.get('next', {}).get('url'))
             if not endpoint:
                 logger.debug('no more pages')
                 break
@@ -117,6 +113,48 @@ class GitHubAPI(RESTclient):
         else:
             return super(GitHubAPI, self).get(endpoint, **kwargs)
 
+    def total(self, endpoint):
+        """ return total number of resources
+        """
+        # logger.debug(f'get total number of resources at endpoint {endpoint}')
+        response = self.get(endpoint, raw_response=True)
+        if response.links:
+            last_url = response.links['last']['url']
+            endpoint = self.get_endpoint_from_url(last_url)
+            items = self.get(endpoint)
+            per_page = GitHubAPI.get_per_page_from_url(last_url)
+            last_page = GitHubAPI.get_page_from_url(last_url)
+            total = per_page * (last_page - 1) + len(items)
+        else:
+            items = response.json()
+            total = len(items)
+        return total
+
+    def get_endpoint_from_url(self, url):
+        """ return endpoint from url
+        """
+        return url.replace(f'https://{self.hostname}', '')
+
+    @staticmethod
+    def get_page_from_url(url):
+        """ get page query parameter form url
+        """
+        regex = r'^.*page=(?P<value>\d+).*$'
+        match = re.match(regex, url)
+        if match:
+            return int(match.group('value'))
+
+    @staticmethod
+    def get_per_page_from_url(url):
+        """ get per_page query parameter from url
+        """
+        per_page = DEFAULT_PAGE_SIZE
+        regex = r'^.*per_page=(?P<value>\d+).*$'
+        match = re.match(regex, url)
+        if match:
+            per_page = int(match.group('value'))
+        return per_page
+
     @classmethod
     def get_client(cls):
         """ return instance of GitHubAPI
